/*  Orphy GTS example2
 *  Copyright (c) Clment 'phneutre' Bourdarias (code)
 *                   email: phneutre@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "orphy.h"
#include <pthread.h>

FILE *log_file=NULL;
OGTS_pref_t acq_prefs;

int my_callback_f (void *param1, void *param2)
{
  OGTS_point_t *point; 
  int i=-1;
  
  point = (OGTS_point_t *)param1;
  i = *((int *)param2);

  OGTS_Printf("(EA%d) t=%f U=%f\n",i,point->x,point->y);

  if (log_file != NULL)
    fprintf(log_file,"%f %f\n",point->x,point->y);

  return 1;
}

int open_logfile()
{
  log_file = fopen("log_file","w");
  if (log_file == NULL)
    OGTS_Error("Error opening log file: %s\n",strerror(errno));
  return 1;
}

int close_logfile()
{
  if (fclose(log_file) != 0)
    OGTS_Printf("Warning: Could not close file\n");
  
  log_file = NULL;
  return 1;
}

/* Thread that will do the acquisition */
void *argument_thread(void *args)
{
  OGTS_StartEA(&acq_prefs,OGTS_TIME,my_callback_f);

  pthread_exit(NULL);
  return(NULL);
}

/* Creates the thread and waits for it to finish.
In a real application, the main thread would do something,
but here we have nothing to do, so we use pthread_join to wait*/
void launch_acquisition()
{
  pthread_t th_tid;

  if (OGTS_CountEAs(&acq_prefs))
    pthread_create(&th_tid, NULL, argument_thread, NULL);

  pthread_join(th_tid,NULL);
}


int main(void)
{
  char car=0;
  int i=0;

  /*OGTS_Enable(OGTS_VERBOSE);*/
  OGTS_OpenDevice(OGTS_SIMUMODE, OGTS_SERIAL0, NULL);
  
  OGTS_Printf("Model: %s\n", OGTS_GetString());
  OGTS_Printf("Using liborphy version %s\n", OGTS_GetVersion());

  /* set acquisition parameters by filling the structure */
  acq_prefs.EA = OGTS_EA0;
  acq_prefs.EF = -1;

  acq_prefs.total_time = 20;
  acq_prefs.dt = 200;
  acq_prefs.numacq = -1;
  acq_prefs.Umax = 4900;
  
  /*open_logfile();*/
  launch_acquisition();
  /*close_logfile();*/

  OGTS_CloseDevice();
  return 1;
}
