#ifndef _OGTS_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>

#include <termios.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <string.h>
#include <math.h>

#define OGTS_VERSION "0.2.1"

#define BAUDRATE        B9600

#define OGTS_QUICKMODE  0
#define OGTS_TEXTMODE   1
#define OGTS_PROGMODE   2
#define OGTS_SIMUMODE   3

#define MAXPRINTMSG	4096

#define OGTS_NUM        0
#define OGTS_TIME       1
#define OGTS_KEY        2

#define OGTS_EA0        1
#define OGTS_EA1        2
#define OGTS_EA2        4
#define OGTS_EA3        8
#define OGTS_EA4        16
#define OGTS_EA5        32
#define OGTS_EA6        64
#define OGTS_EA7        128

#define OGTS_EF0        0
#define OGTS_EF1        1

#define OGTS_SERIAL0    "/dev/ttyS0"
#define OGTS_SERIAL1    "/dev/ttyS1"
#define OGTS_SERIAL2    "/dev/ttyS2"
#define OGTS_SERIAL3    "/dev/ttyS3"
#define OGTS_COM1       "/dev/ttyS0"
#define OGTS_COM2       "/dev/ttyS1"
#define OGTS_COM3       "/dev/ttyS2"
#define OGTS_COM4       "/dev/ttyS3"

#define OGTS_VERBOSE    1
#define OGTS_LOGGING    2

typedef int (*OGTS_callback)(void *param1, void *param2);
typedef void (*OGTS_IO_callback)(int param, siginfo_t *param2, void *param3);

extern int wait_flag;
extern int time_i, time_f;
extern char *device_name;
extern char OGTS_EAs[8];
extern float OGTS_EA_factors[8];

typedef struct OGTS_point_t
{
  float x;
  float y;
} OGTS_point_t;

/* TODO: what if we want more than 1 EA ? */
typedef struct OGTS_pref_t
{
  char EA;        /* EAs or'd between themselves */
  int EF;         /* use EF to start, if >=0 */

  int total_time; /* in milliseconds */
  int dt;         /* in milliseconds */
  
  int numacq;     /* number of acquisitions to do */

  int Umax;       /* in millivolts */

} OGTS_pref_t;

extern char *OGTS_ModelString;

void OGTS_Wait_for_signal(void);
int OGTS_OpenDevice(char mode, char *device, OGTS_IO_callback io_cb);
void OGTS_CloseDevice();
void OGTS_SwitchMode(int mode);
int OGTS_GetStatus();
void OGTS_ResetDevice();
char* OGTS_GetString();

void OGTS_SendCommand(char *cmd);
int OGTS_AskForEA(int num);
int OGTS_ReadOneByte(void);
int OGTS_ReadEA(int num);
int OGTS_ReadEF(int num);
int OGTS_CountEAs(OGTS_pref_t *prefs);

void OGTS_StartEA(OGTS_pref_t *prefs, char mode, OGTS_callback callback);
void OGTS_StopEA();
void OGTS_StartEA2(int EA,int ds,char mode, int param, OGTS_callback callback, void *param2);

void OGTS_Error (char *error, ...);
void OGTS_Printf (char *error, ...);
int OGTS_GetMode();
char *OGTS_GetVersion(void);

int OGTS_GetTicks();
void OGTS_Delay(int ms);

int OGTS_VerifyParam(char what);
void OGTS_Enable(char param);
void OGTS_Disable(char param);
int OGTS_IsEnabled(char param);

void OGTS_signal_handler_IO(int status, siginfo_t *sig_infos, void *param3);
#endif
