/*  Orphy GTS acquisition library (liborphy)
 *  Copyright (c) Clment 'phneutre' Bourdarias
 *                   email: phneutre@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "orphy.h"

struct termios oldtermset, newtermset;
struct timeval start;

int trans_mode = -1;
int OGTS_initialized=0;

char *device_name;
static char *OGTS_ModelString;
int fd;
extern int time_f; extern int time_i;

/*
==================
OGTS_Wait_for_signal
==================
*/
void OGTS_Wait_for_signal(void)
{
  sigset_t mask, oldmask;

  sigemptyset (&mask);
  sigaddset (&mask, SIGIO);

  sigprocmask (SIG_BLOCK, &mask, &oldmask);

#ifdef OGTS_DEBUG
  OGTS_Printf("waiting for signal\n");
#endif  
  while (wait_flag == 1) {
    /* waiting for SIGIO signal */
    sigsuspend (&oldmask);
#ifdef OGTS_DEBUG
    OGTS_Printf("we recieved a signal\n");
#endif
  }
  
#ifdef OGTS_DEBUG
  OGTS_Printf("signal catched, we continue\n");
#endif

  sigprocmask (SIG_UNBLOCK, &mask, NULL); 
}

/*
==================
OGTS_signal_handler_IO
==================
*/
void OGTS_signal_handler_IO(int status, siginfo_t *sig_infos, void *param3)
{
  time_f = OGTS_GetTicks();

#ifdef OGTS_DEBUG
  OGTS_Printf("Recieved SIGIO signal type %s\n",(sig_infos->si_band&POLL_IN)?"POLL_IN":"POLL_OUT");
#endif

  /* OGTS_Printf("Recieved SIGIO signal type %s after %d ms of wait\n",(sig_infos->si_band&POLL_IN)?"POLL_IN":"POLL_OUT",time_f-time_i);*/

  /* we ignore the POLL_OUT signal that appeared in kernels > 2.4.18pre8 */
  if (sig_infos->si_band&POLL_IN) {
    wait_flag = 0;
  }
}

/*
==================
OGTS_OpenDevice
==================
*/
int OGTS_OpenDevice(char mode, char *device, OGTS_IO_callback io_cb)
{
  struct sigaction saio;

  if (OGTS_initialized) {
    OGTS_Printf("(OGTS_OpenDevice): Device already opened, please close it first\n");
    return -1;
  }

  if (mode != OGTS_SIMUMODE) {

    if (setpgrp () < 0)
      {
	OGTS_Error("Error: %s\n",strerror(errno));
      }
    
    /* Set first ticks value */
    gettimeofday(&start, NULL);
    
    device_name = calloc(64,sizeof(char));
    OGTS_Printf("-> Opening device %s\n",device);
    fd = open(device ,O_RDWR|O_NOCTTY|O_NONBLOCK);
    if (fd < 0) {
      OGTS_Error("Error opening device %s: %s\n",device,strerror(errno));
    }
    
    strncpy(device_name,device,64);
  
    /* use user's SIGIO callback if not NULL */
    if (io_cb == NULL)
      saio.sa_sigaction = OGTS_signal_handler_IO;
    else saio.sa_sigaction = io_cb;
    
    //  saio.sa_mask=SIGIO;
    //sigemptyset(&saio.sa_mask);
    saio.sa_flags = SA_SIGINFO;
    saio.sa_restorer = NULL;
    sigaction(SIGIO,&saio,NULL);
    
    /* allow the process to receive SIGIO */
    /*fcntl(fd, F_SETOWN, getpid());*/
    fcntl(fd, F_SETOWN, -getpgrp());
    fcntl(fd, F_SETFL, FASYNC);
    fcntl(fd, F_SETSIG, SIGIO);
    
    /* TODO: check for return values */
    tcgetattr(fd,&oldtermset);
    memset(&newtermset, 0,sizeof(newtermset));
    
    newtermset.c_iflag = IGNPAR|ICRNL;
    newtermset.c_oflag = ONLCR|OPOST;
    newtermset.c_cflag = CS8|CREAD|CLOCAL|B9600;
    
    /* use CANONICAL mode for textmode only */
    switch (mode) {
    case OGTS_QUICKMODE:
      newtermset.c_lflag = 0;
      break;
    case OGTS_TEXTMODE:
    case OGTS_PROGMODE:
      newtermset.c_lflag = ICANON;
      newtermset.c_cc[VMIN]=1;
      newtermset.c_cc[VTIME]=0;
      break;
      
    default:
      break;
    }
    
    cfsetispeed(&newtermset,B9600); 
    cfsetospeed(&newtermset,B9600);
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd,TCSANOW,&newtermset);
    
    switch (mode) {
    case OGTS_QUICKMODE:
      OGTS_SendCommand("XBIN\n");
      break;
      
    case OGTS_TEXTMODE:
    case OGTS_PROGMODE:
      OGTS_SendCommand("XASC\n");
      break;
      
    default:
      OGTS_SendCommand("XASC\n");
      break;
    }
  }

  else fd = 0;
    
  trans_mode = mode;
    
  /* allocate model string */
  OGTS_ModelString = calloc(128,sizeof(char));
  
  OGTS_initialized = 1;
  wait_flag=1;

  return fd;
}

/*
==================
OGTS_CloseDevice
==================
*/
void OGTS_CloseDevice()
{
  if (OGTS_GetMode() != OGTS_SIMUMODE) {

    if (OGTS_initialized) {
      OGTS_Printf("<- Closing device %s\n",device_name);
      
      /* reload initial settings for serial port */
      tcsetattr(fd,TCSANOW,&oldtermset);
      
      if (close(fd) <0) {
	OGTS_Error("Error closing device %s: %s\n",device_name,strerror(errno));
      }
      
      free(OGTS_ModelString);
      OGTS_initialized =0;
    }
    
    else OGTS_Printf("(OGTS_CloseDevice): Device not opened\n");
  }
}

/*
==================
OGTS_SwitchMode

Switches between Quickmode and Textmode
==================
*/
void OGTS_SwitchMode(int mode)
{
  if ( (OGTS_initialized == 0) || (OGTS_GetMode() == OGTS_SIMUMODE) ) return;

  switch (mode) {
  case OGTS_QUICKMODE:
    newtermset.c_lflag = 0;
    newtermset.c_cc[VMIN]=0;
    newtermset.c_cc[VTIME]=0;
    break;
  case OGTS_TEXTMODE:
  case OGTS_PROGMODE:
    newtermset.c_lflag = ICANON;
    newtermset.c_cc[VMIN]=1;
    newtermset.c_cc[VTIME]=0;
    break;

  default:
    break;
  }

  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtermset);

  switch (mode) {
  case OGTS_QUICKMODE:
    OGTS_SendCommand("XBIN\n");
    break;

  case OGTS_TEXTMODE:
  case OGTS_PROGMODE:
    OGTS_SendCommand("XASC\n");
    break;

  default:
    OGTS_SendCommand("XASC\n");
    break;
  }

  trans_mode = mode;
}

/*
==================
OGTS_GetMode
==================
*/
int OGTS_GetMode()
{
  return trans_mode;
}

/*
==================
OGTS_GetString

Returns the model name
==================
*/
char* OGTS_GetString()
{
  char *answer = calloc(64,sizeof(char));
  char *dash=NULL;
  char data=0;
  int i=0;
  int need_to_switch_back=0;

  wait_flag=1;

  switch (trans_mode) {
  case OGTS_TEXTMODE:
    OGTS_SendCommand("XERR\n");
    break;

  case OGTS_QUICKMODE: 
    OGTS_SwitchMode(OGTS_TEXTMODE);
    OGTS_SendCommand("XERR\n");
    need_to_switch_back=1;
    break;

  default: break;
  }

  if (OGTS_GetMode() != OGTS_SIMUMODE) {

    time_i = OGTS_GetTicks();
    
    OGTS_Wait_for_signal();
    
    /*while (wait_flag == 1) {
      
    pause();
    }*/
    
    while (i < 64) {
      if (data == 10 && i != 1) break;
      if (read(fd,&data,1) < 0) {
	OGTS_Error("Error reading on device %s: %s",device_name,strerror(errno));
      }
      /*printf("data=%c (%d)\n",data,data);*/
      answer[i] = data;
      i++;
    }
    
    wait_flag=1;
    
    if (need_to_switch_back)
      OGTS_SwitchMode(OGTS_QUICKMODE);

    dash = strchr(answer,'-');
    memset(dash,0,dash-answer+1);
    strncpy(OGTS_ModelString,answer+1,128);

    free(answer);
  }

  else  strcpy(OGTS_ModelString, "FAKE DEVICE (simulation mode)");

  return (OGTS_ModelString);
}

/*
==================
OGTS_GetStatus
==================
*/
int OGTS_GetStatus()
{
  char answer[64];
  char data=0;
  int i=0;

  memset(answer,0,63);
  wait_flag=1;

  switch (trans_mode) {
  case OGTS_TEXTMODE:
    OGTS_SendCommand("XERR\n");
    break;

  default: return -1; 
    break;
  }

  time_i = OGTS_GetTicks();

  OGTS_Wait_for_signal();

  /*  while (wait_flag == 1) {
    pause();
    }*/

  while (i < 64) {
    if (data == 10 && i != 1) break;
    if (read(fd,&data,1) < 0) {
      OGTS_Error("Error reading on device %s: %s",device_name,strerror(errno));
    }
    //printf("data=%c (%d)\n",data,data);
    answer[i] = data;
    i++;
  }

  wait_flag=1;
  OGTS_Printf("<- Orphy says: %s\n",answer);
  return 1;
}

/*
==================
OGTS_ResetDevice
==================
*/
void OGTS_ResetDevice()
{
  if ((OGTS_initialized) && (OGTS_GetMode() != OGTS_SIMUMODE))
    OGTS_SendCommand("XRESET\n");
}
