/*  Orphy GTS acquisition library (liborphy)
 *  Copyright (c) Clment 'phneutre' Bourdarias
 *                   email: phneutre@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "orphy.h"

extern struct timeval start;

/*
==================
OGTS_GetTicks
==================
*/
int OGTS_GetTicks(void)
{
  struct timeval now;
  int ticks;

  gettimeofday(&now, NULL);
  ticks=(now.tv_sec-start.tv_sec)*1000+(now.tv_usec-start.tv_usec)/1000;
  return(ticks);
}

/*
==================
OGTS_Delay
==================
*/
void OGTS_Delay(int ms)
{
  struct timeval tv;
  int then, now, elapsed;
  int was_error;

  tv.tv_sec = ms/1000;
  tv.tv_usec = (ms%1000)*1000;

  then = OGTS_GetTicks();

  do {
    errno = 0;

    now = OGTS_GetTicks();
    elapsed = (now-then);
    then = now;
    if ( elapsed >= ms ) {
      break;
    }
    ms -= elapsed;
    tv.tv_sec = ms/1000;
    tv.tv_usec = (ms%1000)*1000;
    
    was_error = select(0, NULL, NULL, NULL, &tv);
  } while ( was_error && (errno == EINTR) );
}
